<?php

namespace Blesta\Core\Util\Tax;

use Blesta\Core\Util\Tax\Common\AbstractTax;
use PH7\Eu\Vat\Validator;
use PH7\Eu\Vat\Provider\Europa;
use Configure;
use Loader;

/**
 * European Union Tax
 *
 * @package blesta
 * @subpackage blesta.core.Util.Tax
 * @copyright Copyright (c) 2020, Phillips Data, Inc.
 * @license http://www.blesta.com/license/ The Blesta License Agreement
 * @link http://www.blesta.com/ Blesta
 */
class EuropeTax extends AbstractTax
{
    /**
     * @var array An array of errors
     */
    private $errors = [];

    /**
     * Verifies if a given tax ID is valid
     *
     * @param string $tax_id The tax ID to verify
     * @param array $client An array containing the client data associated with the tax ID
     * @return bool True if the given tax ID is valid, false otherwise
     */
    public function validateTaxId($tax_id, $client = null)
    {
        $country = isset($client['country']) ? $client['country'] : null;

        if (in_array($country, $this->getCountries())) {
            try {
                $validator = new Validator(new Europa, $tax_id, $country == 'GR' ? 'EL' : $country);

                return $validator->check();
            } catch (\Throwable $e) {
                $this->errors[] = ['valid' => $e->getMessage()];
            }
        }

        return false;
    }

    /**
     * Gets the tax information of a given tax ID
     *
     * @param string $tax_id The tax ID from which to obtain the information
     * @param array $client An array containing the client data associated with the tax ID
     * @return array An array containing the tax data
     *
     *  - name The name of the taxpayer
     *  - address The address of the taxpayer
     *  - country The country of the taxpayer
     *  - state The state/province of the taxpayer
     *  - type Whether the taxpayer is an individual or a business
     *  - tax_id The formatted tax ID
     *  - tax_exempt True if the taxpayer is exempt from taxes, false otherwise
     *  - settings A list of additional parameters (optional)
     */
    public function getTaxInformation($tax_id, $client = null)
    {
        Loader::loadModels($this, ['Companies']);

        $country = isset($client['country']) ? $client['country'] : null;

        if (in_array($country, $this->getCountries())) {
            if (empty($tax_id)) {
                // The taxpayer is an individual
                return [
                    'name' => (isset($client['first_name']) ? $client['first_name'] : '')
                        . (isset($client['last_name']) ? ' ' . $client['last_name'] : ''),
                    'address' => isset($client['address1']) ? $client['address1'] : '',
                    'country' => $country,
                    'type' => 'individual',
                    'tax_id' => $tax_id,
                    'tax_exempt' => false
                ];
            } else {
                // The taxpayer is a business
                try {
                    $validator = new Validator(new Europa, $tax_id, $country == 'GR' ? 'EL' : $country);

                    $tax_home = $this->Companies->getSetting(Configure::get('Blesta.company_id'), 'tax_home_eu_vat');
                    $tax_home = isset($tax_home->value) ? $tax_home->value : null;

                    $tax_exempt = false;
                    if ($validator->check() && $validator->getCountryCode() !== $tax_home) {
                        $tax_exempt = true;
                    }

                    return [
                        'name' => $validator->getName(),
                        'address' => $validator->getAddress(),
                        'country' => $validator->getCountryCode(),
                        'type' => 'business',
                        'tax_id' => $validator->getVatNumber(),
                        'tax_exempt' => $tax_exempt
                    ];
                } catch (\Throwable $e) {
                    $this->errors[] = ['valid' => $e->getMessage()];
                }
            }
        }

        return [
            'name' => (isset($client['first_name']) ? $client['first_name'] : '')
                . (isset($client['last_name']) ? ' ' . $client['last_name'] : ''),
            'address' => isset($client['address1']) ? $client['address1'] : '',
            'country' => isset($client['country']) ? $client['country'] : '',
            'type' => (empty($client['company']) ? 'individual' : 'business'),
            'tax_id' => $tax_id,
            'tax_exempt' => true
        ];
    }

    /**
     * Gets a list of the countries where these tax requirements apply
     *
     * @return array A list containing the country codes in ISO 3166-1 alpha2
     */
    public function getCountries()
    {
        return [
            'AT', 'BE', 'BG', 'CY', 'CZ',
            'DE', 'DK', 'EE', 'ES', 'FI',
            'FR', 'GB', 'GR', 'HU', 'HR',
            'IE', 'IT', 'LT', 'LU', 'LV',
            'MT', 'NL', 'PL', 'PT', 'RO',
            'SE', 'SI', 'SK'
        ];
    }

    /**
     * Retrieves any errors associated with the tax api
     *
     * @return array An array of errors
     */
    public function errors()
    {
        return $this->errors;
    }
}